#####################################################################################
#
#		:::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]::: 
#      ____                __                                                      
#     / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __   
#    / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // /   
#   /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/    
#        /___/ /___/                 /_/    /_/           /___/ /_/                
#
#
#####################################################################################
#
# servinfo 1.2
#
# Description:
#	Displays http server information
#
# Authors: Shrike <shrike@eggdrop.org.ru>
#
# Official support: irc.eggdrop.org.ru @ #eggdrop
# 
#####################################################################################
#
# Version History:
#	
#  		SHR - Shrike
#	
#		Who		DATE	Changes
#   	---     ------  -------------------------------------------------------------
# v1.2
#   	SHR		040618	Initial public version.
#
#####################################################################################
# Last version of egglib_pub can be grabbed here:
# http://eggdrop.org.ru/scripts/egglib.zip

if { ![info exists egglib(ver)] || [expr {$egglib(ver) < 1.2}] } {
	putlog "***********************************************"
	putlog "   egglib_pub NOT FOUND OR VERSION IS TOO OLD!"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"
	die
}

foreach p [array names servinfo *] { catch { putlog "unsetting servinfo($p)"; unset servinfo($p) } }

namespace eval servinfo {}

bind pub - !server ::servinfo::pub
bind msg - !server ::servinfo::msg
bind pub - ! ::servinfo::pub
bind msg - ! ::servinfo::msg

setudef flag nopubserver

set servinfo(savetofile) 0
set servinfo(file) "~/server_answers.txt"
set servinfo(agent) "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 1.0.3705)"
set servinfo(timeout) 10
set servinfo(ver) "1.2"
set servinfo(authors) "Shrike <shrike@eggdrop.org.ru>"

proc ::servinfo::msg {nick uhost hand args} {
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	::servinfo::servinfo $nick $uhost $hand $nick $args
}

proc ::servinfo::pub {nick uhost hand chan args} {
	if {[channel get $chan nopubserver]} return
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	::servinfo::servinfo $nick $uhost $hand $chan $args
}

proc ::servinfo::servinfo {nick uhost hand chan args} {
	global servinfo
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	set addr [lindex $args 0]
	set host [lindex [split $addr :] 0]; set port [lindex [split $addr :] 1]
	if { $port == "" } { set port 80 }
	::egglib::log $chan $nick "servinfo" "$host:$port"
	if { [info exists servinfo(host,$uhost)] } { ::egglib::outn $nick "    "; return }
	
	if {$host == ""} { ::egglib::outh $nick $chan "!server" "<host>\[:port\]"; return }
	set servinfo(host,$uhost) 1

	set url "$host:$port/"
	set id [::egglib::http_init "::servinfo::"]
	::egglib::http_set_timeout $id $servinfo(timeout)
	::egglib::http_set_method $id "HEAD"
	::egglib::http_set_agent $id $servinfo(agent)
  	::egglib::http_get $id $url [list $nick $chan $uhost $host $port]
}

proc ::servinfo::on_error { id nick chan uhost host port} {
	global servinfo
	catch {unset servinfo(host,$uhost)}
	::egglib::out $nick $chan "\0032  \003 [::egglib::host_color "$host:$port"]"
}

proc ::servinfo::on_data { id html nick chan uhost host port } {
	global servinfo
	set info ""
	catch {unset servinfo(host,$uhost)}
	foreach line [split $html \n] { 
		if { [regexp -nocase -- {^Server\:(.*)$} $line garb info] } { break }	
	}
	set info [string trim $info]
	if { $info == "" } {
		::egglib::out $nick $chan "\0032   \003 [::egglib::host_color "$host:$port"]"
	} else {
		::egglib::out $nick $chan "[::egglib::host_color "$host:$port"]\0032 - [::egglib::version_color $info]\003"
		if { $servinfo(savetofile) } { ::servinfo::add_answer $info	}
	}
}

proc ::servinfo::read_list { } { global servinfo; set servinfo(list) [::egglib::readdata $servinfo(file)] }
proc ::servinfo::write_list { } { global servinfo; ::egglib::writedata $servinfo(file) $servinfo(list) }

proc ::servinfo::add_answer {info} {
	global servinfo	
	::servinfo::read_list
	foreach i $servinfo(list) { if { [string equal $info $i] } { return } }
	lappend servinfo(list) "$info"
	::servinfo::write_list
	catch { unset servinfo(list) }
}

putlog "servinfo.tcl $servinfo(ver) by $servinfo(authors) loaded"